import os
import csv
import datetime

import arcpy

# Author J.Connolly

# Description: Copies the files from the geodatabase and re-names to the standard file name schema used a dictionary
# look-up variable; NmChangeDICT.  This dictionary is create manually using the master species list.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

# ###Varibles to be set by user
# Input File Locations
# Set the workspace for the ListFeatureClass function
# Gdb with copied files from step 2
InGDB = r"path\filename.gdb"
abb = "FWS"  # FWS or NMFS

# Workspace for copied and standardize files; stays static for the whole tool
ws = "path"
# Folder in workspace where outputs will be saved
name_dir = "folder name"

# in yyyymmdd received date
receivedDate = 'date'

# DICT for name change original file to EPA Standard name
# Note date must be in in yyyymmdd in file name

NmChangeDICT = {'original file name': 'STD filename_yyyymmdd ',
                }


# FUNCTIONS
def fcs_in_workspace(workspace):
    # recursively checks workspaces found for feature classes
    arcpy.env.workspace = workspace
    for fc in arcpy.ListFeatureClasses():
        yield (fc)
    for wsp in arcpy.ListWorkspaces():
        for fc in fcs_in_workspace(wsp):
            yield fc


def create_directory(path_directory, out_csv, out_gdb):
    # creates directories to save files
    if not os.path.exists(path_directory):
        os.mkdir(path_directory)
        print "created directory {0}".format(path_directory)
    if not os.path.exists(out_csv):
        os.mkdir(out_csv)
        print "created directory {0}".format(out_csv)
    if not os.path.exists(out_gdb):
        os.mkdir(out_gdb)
        print "created directory {0}".format(out_gdb)


def create_flnm_timestamp(namefile, outlocation, date_list, file_extension):
    # creates date stamped generic file
    file_extension.replace('.', '')
    filename = str(namefile) + "_" + str(date_list[0]) + '.' + file_extension
    filepath = os.path.join(outlocation, filename)
    return filename, filepath


def create_out_table(list_name, csv_name):
    # outputs table from list generated in create FileList
    with open(csv_name, "wb") as output:
        writer = csv.writer(output, lineterminator='\n')
        for val in list_name:
            writer.writerow([val])


def create_gdb(out_folder, out_name, outpath):
    # Create a new GDB
    if not arcpy.Exists(outpath):
        arcpy.CreateFileGDB_management(out_folder, out_name, "CURRENT")


def stand_name(in_gdb, out_file_gdbpath, out_file_fail_gdbpath):
    # loops through in_gdb and makes a copy of each file applying the std filename from the dict in outGDB
    for fc in fcs_in_workspace(in_gdb):
        try:
            dsc = arcpy.Describe(fc)
            sr = dsc.spatialReference
            new_name = NmChangeDICT[fc]
            add_sr_list = str(fc) + "," + "Name: " + sr.name + "," + "Type: " + sr.type + "," + str(new_name)
            print add_sr_list
            OrgSRList.append(add_sr_list)
            out_fc = os.path.join(out_file_gdbpath, new_name)
            # print out_fc
            if not arcpy.Exists(out_fc):
                # print "FC does not exist"
                arcpy.CopyFeatures_management(fc, out_fc)
            else:
                print" FC already exists"
        except:
            print "Failed  " + str(fc)
            add_failed = str(fc)
            FailedList.append(add_failed)
            out_failed_fc = os.path.join(out_file_fail_gdbpath, add_failed)
            arcpy.CopyFeatures_management(fc, out_failed_fc)


# static variable no user input needed unless changing code structure
datelist = []
today = datetime.date.today()
datelist.append(today)

OrgSRList = []
FailedList = []
addSRList = "Filename Original (GDB)" + "," + "Original Projection" + "," + "Original Projection Type" + "," + "Standardize Filename"
OrgSRList.append(addSRList)
addFailed = "Filename-Original (GDB)"
FailedList.append(addFailed)

path_dir = ws + os.sep + str(name_dir)
outLocationCSV = path_dir + os.sep + "CSV"
OutFolderGDB = path_dir + os.sep + "GDB"

# Output File Names
ReNmCSVCopied = 'ReNm' + str(abb) + "_" + str(receivedDate)
out_nameGDB = "ReNm_" + str(abb) + "_" + str(receivedDate)
FailedGDB = "Failed_" + str(out_nameGDB)
ReNmCSVFailed = "Failed_" + str(ReNmCSVCopied)

# CREATES FILE NAMES
# CSV out table succeed and failed
csvfile, csvpath = create_flnm_timestamp(ReNmCSVCopied, outLocationCSV, datelist, 'csv')
failedcsv, failedcsvpath = create_flnm_timestamp(ReNmCSVFailed, outLocationCSV, datelist, 'csv')
# GDB succeed and failed
OutGDB, outFilegdbpath = create_flnm_timestamp(out_nameGDB, OutFolderGDB, datelist, 'gdb')
FailGDB, outFilefailgdbpath = create_flnm_timestamp(FailedGDB, OutFolderGDB, datelist, 'gdb')

arcpy.env.scratchWorkspace = ""
# NOTE Change this to False if you don't want GDB to be overwritten
arcpy.env.overwriteOutput = True

# Start script

# Copy shapefiles to a file geodatabase and rename

# start clock on timing script

start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

create_directory(path_dir, outLocationCSV, OutFolderGDB)

create_gdb(OutFolderGDB, OutGDB, outFilegdbpath)
create_gdb(OutFolderGDB, FailGDB, outFilefailgdbpath)
stand_name(InGDB, outFilegdbpath, outFilefailgdbpath)

# ##write data store in lists to out tables in csv format
create_out_table(OrgSRList, csvpath)
create_out_table(FailedList, failedcsvpath)
# #End clock time script
end = datetime.datetime.now()
print "End Time: " + end.ctime()

elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
